﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace LessonX
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }
        // フォームロード時
        private void frmMain_Load(object sender, EventArgs e)
        {
            string sid = Application.ProductName + Application.ProductVersion;

            // JVLink初期化
            long ret = axJVLink1.JVInit(sid);

            // エラー判定
            if (ret != 0)
            {
                MessageBox.Show("JVInit エラー コード：" + ret + "：", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        // メニュー：JV-Linkの設定 押下時
        private void menuConfJv_Click(object sender, EventArgs e)
        {
            try
            {
                long ret;

                ret = axJVLink1.JVSetUIProperties();
                if (ret != 0)
                {
                    MessageBox.Show("JVSetUIProperties()エラーコード：" + ret + "：", Application.ProductName,  MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show("JVSetUIProperties() 例外：" + ex.Message + "：", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        // データ取得ボタン押下時
        private void btnGetJVData_Click(object sender, EventArgs e)
        {
            long ret;
            string dataSpec;
            string fromTime;
            int option;
            int readCount = 0;
            int downloadCount = 0;
            string lastFileTimeSttamp;

            const int buffSize = 110000;    // JV
            const int nameSize = 256;
            string buff;
            string fileName;
            var raceInfo = new JVData_Struct.JV_RA_RACE();

            // 引数設定
            dataSpec = "RACE";             // レース情報
            fromTime = "00000000000000";   // 今週データ
            option = 2;

            try
            {
                // JVLinkダウンロード処理
                ret = axJVLink1.JVOpen(dataSpec, fromTime, option, ref readCount, ref downloadCount, out lastFileTimeSttamp);

                // エラー判定
                if (ret != 0)
                {
                    MessageBox.Show("JVOpen()の戻り値：" + ret + "：", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show("JVOpen()の戻り値      ：" + ret + Environment.NewLine +
                                    "読み込みファイル数    ：" + readCount + Environment.NewLine +
                                    "ダウンロードファイル数：" + downloadCount + Environment.NewLine +
                                    "タイムスタンプ        ：" + lastFileTimeSttamp, 
                                     Application.ProductName, 
                                     MessageBoxButtons.OK);


                    if (readCount > 0)
                    {
                        bool flag = true;
                        do
                        {
                            buff = new string('\0', buffSize);
                            fileName = new string('\0', nameSize);

                            // JVReadで１行読み込み
                            int readSize = buffSize;
                            ret = axJVLink1.JVRead(out buff, out readSize, out fileName);

                            // リターンコードにより処理を分岐
                            switch (ret)
                            {
                                case 0:  // 全ファイル読み込み終了
                                    flag = false;
                                    break;
                                case -1:  // ファイル切り替わり
                                    break;
                                case -3:  // ダウンロード中
                                    break;
                                case -20: // Initされていない
                                    MessageBox.Show("JVInit()が行われていません。",
                                                     Application.ProductName,
                                                     MessageBoxButtons.OK);
                                    flag = false;
                                    break;
                                case -203:  // Openされていない
                                    MessageBox.Show("JVOpen()が行われていません。",
                                                     Application.ProductName,
                                                     MessageBoxButtons.OK);
                                    flag = false;
                                    break;
                                case -503:  // Openされていない
                                    MessageBox.Show(fileName + "が存在しません。",
                                                     Application.ProductName,
                                                     MessageBoxButtons.OK);
                                    flag = false;
                                    break;
                                default:
                                    if (ret > 0)
                                    {
                                        if (buff.Substring(0, 2) == "RA")
                                        {
                                            raceInfo.SetDataB(ref buff);

                                            // データ表示
                                            rtbData.AppendText(
                                                "年：" + raceInfo.id.Year + 
                                                " 月日：" + raceInfo.id.MonthDay +
                                                " 場：" + raceInfo.id.JyoCD +
                                                " 回次：" + raceInfo.id.Kaiji +
                                                " 日次：" + raceInfo.id.Nichiji +
                                                " Ｒ：" + raceInfo.id.RaceNum +
                                                " レース名：" + raceInfo.RaceInfo.Ryakusyo10 + Environment.NewLine
                                                );
                                        }
                                    }
                                    break;
                            }

                        } while (flag);
                    }





                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + "：", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            // JVLink終了処理
            ret = axJVLink1.JVClose();

            // エラー判定
            if (ret != 0)
            {
                MessageBox.Show("JVClose()の戻り値：" + ret + "：", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                MessageBox.Show("処理が正常に完了しました。",
                                 Application.ProductName,
                                 MessageBoxButtons.OK);
            }
        }

    }
}
